package sun.font;

import java.awt.font.FontRenderContext;
import java.awt.Font;

final class StandardTextSource extends TextSource
{
    private final char[] chars;
    private final int start;
    private final int len;
    private final int cstart;
    private final int clen;
    private final int level;
    private final int flags;
    private final Font font;
    private final FontRenderContext frc;
    private final CoreMetrics cm;
    
    StandardTextSource(char[] array, int i, int j, int n, int k, int level, int flags, Font font, FontRenderContext fontRenderContext, CoreMetrics cm) {
        if (array == null) {
            throw new IllegalArgumentException("bad chars: null");
        }
        if (n < 0) {
            throw new IllegalArgumentException("bad cstart: " + n);
        }
        if (i < n) {
            throw new IllegalArgumentException("bad start: " + i + " for cstart: " + n);
        }
        if (k < 0) {
            throw new IllegalArgumentException("bad clen: " + k);
        }
        if (n + k > array.length) {
            throw new IllegalArgumentException("bad clen: " + k + " cstart: " + n + " for array len: " + array.length);
        }
        if (j < 0) {
            throw new IllegalArgumentException("bad len: " + j);
        }
        if (i + j > n + k) {
            throw new IllegalArgumentException("bad len: " + j + " start: " + i + " for cstart: " + n + " clen: " + k);
        }
        if (font == null) {
            throw new IllegalArgumentException("bad font: null");
        }
        if (fontRenderContext == null) {
            throw new IllegalArgumentException("bad frc: null");
        }
        this.chars = array;
        this.start = i;
        this.len = j;
        this.cstart = n;
        this.clen = k;
        this.level = level;
        this.flags = flags;
        this.font = font;
        this.frc = fontRenderContext;
        if (cm != null) {
            this.cm = cm;
        }
        else {
            this.cm = ((FontLineMetrics)font.getLineMetrics(array, n, k, fontRenderContext)).cm;
        }
        return;
    }
    
    @Override
    public char[] getChars() {
        return this.chars;
    }
    
    @Override
    public int getStart() {
        return this.start;
    }
    
    @Override
    public int getLength() {
        return this.len;
    }
    
    @Override
    public int getContextStart() {
        return this.cstart;
    }
    
    @Override
    public int getContextLength() {
        return this.clen;
    }
    
    @Override
    public int getLayoutFlags() {
        return this.flags;
    }
    
    @Override
    public int getBidiLevel() {
        return this.level;
    }
    
    @Override
    public Font getFont() {
        return this.font;
    }
    
    @Override
    public FontRenderContext getFRC() {
        return this.frc;
    }
    
    @Override
    public CoreMetrics getCoreMetrics() {
        return this.cm;
    }
    
    @Override
    public TextSource getSubSource(int i, int j, int n) {
        if (i < 0 || j < 0 || i + j > this.len) {
            throw new IllegalArgumentException("bad start (" + i + ") or length (" + j + ")");
        }
        int level = this.level;
        if (n != 2) {
            int n2 = (this.flags & 0x8) == 0x0;
            if ((n != 0 || n2 == 0) && (n != 1 || n2)) {
                throw new IllegalArgumentException("direction flag is invalid");
            }
            level = (n2 ? false : true);
        }
        return new StandardTextSource(this.chars, this.start + i, j, this.cstart, this.clen, level, this.flags, this.font, this.frc, this.cm);
    }
    
    @Override
    public String toString() {
        return this.toString(true);
    }
    
    @Override
    public String toString(boolean b) {
        StringBuffer sb = new StringBuffer(super.toString());
        sb.append("[start:");
        sb.append(this.start);
        sb.append(", len:");
        sb.append(this.len);
        sb.append(", cstart:");
        sb.append(this.cstart);
        sb.append(", clen:");
        sb.append(this.clen);
        sb.append(", chars:\"");
        int i;
        int n;
        if (b) {
            i = this.cstart;
            n = this.cstart + this.clen;
        }
        else {
            i = this.start;
            n = this.start + this.len;
        }
        for (; i < n; ++i) {
            if (i > i) {
                sb.append(" ");
            }
            sb.append(Integer.toHexString((int)this.chars[i]));
        }
        sb.append("\"");
        sb.append(", level:");
        sb.append(this.level);
        sb.append(", flags:");
        sb.append(this.flags);
        sb.append(", font:");
        sb.append(this.font);
        sb.append(", frc:");
        sb.append(this.frc);
        sb.append(", cm:");
        sb.append(this.cm);
        sb.append("]");
        return sb.toString();
    }
}
